<?php

/*
 * Metafox - enterprise content management system
 *
 * db.inc.php - MySQL query interface
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@platon.sk>
 * Copyright (c) 2001-2008 Platon Group, http://platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://platon.sk/projects/Metafox/
 */

/* $Platon: Metafox/db.inc.php,v 1.43 2007-11-12 16:12:55 nepto Exp $ */

/*
 * updates: 23/5/2001, 29/7/2001, 11/10/2001, 2/11/2001, 11/1/2002, 23/7/2002
 *   whole august/2002, 30/11/2002, 15/1/2003
 */

require_once 'cfgldr.inc.php';
require_once 'functions.inc.php';
require_once 'db-core/db-core.inc.php';

/*
 * VALIDATION FUNCTIONS
 */

/*
 * ezin_validate_IDs()
 */
function ezin_validate_IDs($ar) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$ezin_authors  = $ezin_sys['db']['tables']['authors'];
	$ezin_sections = $ezin_sys['db']['tables']['sections'];
	$ezin_articles = $ezin_sys['db']['tables']['articles'];
	$ezin_messages = $ezin_sys['db']['tables']['messages'];
	$row           = false;
	$ret_row       = false;
	$id_included   = 0;
	$c_author      = ! empty($ar['author_id']);
	$c_section     = ! empty($ar['section_id']);
	$c_article     = ! empty($ar['article_id']);
	$c_message     = ! empty($ar['message_id'])
		&& ! is_array($ar['message_id'])
		&& intval($ar['message_id']) > 0;
	$conditions  = array();
	$tables      = array();
	$select_cols = array();

	if ($c_author || $c_section || $c_article || $c_message) {
		if ($c_author) {
			if ($c_article) {
				$tables[]      = $ezin_articles;
				$select_cols[] = $ezin_articles.'.author_id AS author_id';
				$select_cols[] = $ezin_articles.'.id AS article_id';
				$select_cols[] = $ezin_articles.'.access_key AS article_access_key';
				//	$conditions[]  = $ezin_articles.'.author_id = '.$ezin_authors.'.id';
			} else {
				$tables[]      = $ezin_authors;
				$select_cols[] = $ezin_authors.'.id AS author_id';
				$select_cols[] = $ezin_authors.'.access_key AS author_access_key';
				if (intval($ar['author_id']) > 0) {
					$conditions[] = $ezin_authors.'.id = '.intval($ar['author_id']);
				} else {
					$conditions[] = $ezin_authors.'.access_key = "'.ezin_db_escape($ar['author_id']).'"';
				}
			}
		} else {
			$tables      = array($ezin_sections);
			$select_cols[] = $ezin_sections.'.id AS section_id';
			$select_cols[] = $ezin_sections.'.access_key AS section_access_key';
			$select_cols[] = $ezin_sections.'.bind_lang AS section_bind_lang';
			$select_cols[] = $ezin_sections.'.lang AS lang';

			if ($c_article || $c_message) {
				$tables[]      = $ezin_articles;
				$select_cols[] = $ezin_articles.'.id AS article_id';
				$select_cols[] = $ezin_articles.'.access_key AS article_access_key';
				$select_cols[] = $ezin_articles.'.bind_lang AS article_bind_lang';
				$conditions    = array_merge($conditions, array(
							$ezin_sections.'.id = '.$ezin_articles.'.section_id',
							//$ezin_articles.'.datetime <= SYSDATE() ',
							$ezin_articles.'.status NOT IN ("", "disabled")',
							$ezin_sections.'.type NOT IN ("", "disabled")'
							));
			}

			if ($c_message) {
				$tables[]      = $ezin_messages;
				$select_cols[] = $ezin_messages.'.id AS message_id';
				$conditions[]  = $ezin_articles.'.id = '.$ezin_messages.'.article_id';
				$conditions[]  = $ezin_messages.'.id = '.intval($ar['message_id']);
				$id_included   = 1;
			}
		}

		if (! $id_included && $c_article) {
			if (intval($ar['article_id']) > 0) {
				$conditions[] = $ezin_articles.'.id = '.intval($ar['article_id']);
			} else {
				$conditions[] = $ezin_articles.'.access_key = "'.ezin_db_escape($ar['article_id']).'"';
			}
			$id_included   = 1;
		}

		if (! $id_included && $c_section) {
			$conditions[] = $ezin_sections.'.type IN ("normal", "global", "hidden", "hidden2")'; // )
			if (intval($ar['section_id']) > 0) {
				$conditions[] = $ezin_sections.'.id = '.intval($ar['section_id']);
			} else {
				$conditions[] = $ezin_sections.'.access_key = "'.ezin_db_escape($ar['section_id']).'"';
			}
			$id_included   = 1;
		}

		$query = sprintf('SELECT %s FROM %s WHERE %s',// LIMIT 1',
				join(', ', $select_cols),
				join(', ', $tables),
				join(' AND ', $conditions));
		//echo $query;exit;
		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_validate_IDs()');
		$first   = true;
		$ret_row = false;
		while (($row = @mysql_fetch_array($result, MYSQL_ASSOC)) !== false) {
			if ($first) {
				$first   = false;
				$ret_row = $row;
				if (@$ar['lang'] == false || @$ar['lang'] == '')
					break;
			}
			if (isset($row['lang']) && ! @strcasecmp($row['lang'], $ar['lang'])) {
				$ret_row = $row;
				break;
			}
		}
		@mysql_free_result($result);
	}

	if ($ret_row !== false) {
		$ar['lang']       = @$ret_row['lang'];
		$ar['author_id']  = @$ret_row['author_id'];
		$ar['section_id'] = @$ret_row['section_id'];
		$ar['article_id'] = isset($ret_row['article_id']) ? $ret_row['article_id'] : 0;
		$ar['message_id'] = isset($ret_row['message_id'])
			? $ret_row['message_id']
			: is_array($ar['message_id']) ? $ar['message_id'] : 0;
		$ar['section_bind_lang'] = intval(@$ret_row['section_bind_lang']);
		$ar['article_bind_lang'] = intval(@$ret_row['article_bind_lang']);
	} else {
		$ar['lang']       = '';
		$ar['author_id']  = 0;
		$ar['section_id'] = 0;
		$ar['article_id'] = 0;
		$ar['message_id'] = 0;
		$ar['section_bind_lang'] = 0;
		$ar['article_bind_lang'] = 0;
	}

	return $ar;
} /* }}} */

function ezin_get_bind_lang_table_wrapper(&$lang_tbl_type, $section_bind, $article_bind = 0) /* {{{ */
{
	$lang_tbl_type  = 'A';
	$bind_lang = intval($article_bind);
	if ($article_bind == 0) {
		$lang_tbl_type = 'S';
		$bind_lang = intval($section_bind);
	}
	return ezin_get_bind_lang_table($bind_lang, $lang_tbl_type);
} /* }}} */

function ezin_get_bind_lang_table($bind_lang, $type = 'S' /* S or A */) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$bind_lang = intval($bind_lang);
	if ($bind_lang == 0) {
		return array();
	}

	if ($type == 'S') {
		$query = 'SELECT id, access_key, lang'
			.' FROM '.$ezin_sys['db']['tables']['sections']
			.' WHERE bind_lang = '.$bind_lang
			.' ORDER BY id ASC';
	} else {
		$query = 'SELECT articles.id, articles.access_key, sections.lang FROM '
			.$ezin_sys['db']['tables']['articles'].' AS articles, '
			.$ezin_sys['db']['tables']['sections'].' AS sections '
			.' WHERE articles.section_id = sections.id'
			.' AND articles.bind_lang = '.$bind_lang
			.' ORDER BY articles.datetime ASC';
	}

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_bind_lang_table()');

	$array = array();
	while (($row = mysql_fetch_array($result, MYSQL_NUM)) !== false) {
		$array[$row[2]]     = $row[0];
		$array[$row[2].'2'] = empty($row[1]) ? $row[0] : $row[1];
	}
	@mysql_free_result($result);

	return $array;
} /* }}} */

/*
 * DATA RETRIEVAL FUNCTIONS
 */

/*
 * ezin_get_section_list()
 */ 
function ezin_get_section_list($lang = '', $current_section_id = null) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$current_section_id = intval($current_section_id);
	$query = 'SELECT'
		.' id section_id,'
		.' type section_type,'
		.' ord section_ord,'
		.' a_limit section_limit,'
		.' name section_name,'
		.' access_key   section_access_key,'
		.' template_key section_template_key,'
		.' link section_link'
		.' FROM '.$ezin_sys['db']['tables']['sections']
		.' WHERE type IN ("normal", "global")';
	if ($lang != '') {
		$query .= ' AND (lang IS NULL OR lang = ""'
				.' OR lang LIKE "'.ezin_db_escape($lang).'")';
	}
	$query .= ' ORDER BY ord ASC, type ASC, id ASC';
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_section_list()');
	$array = array();
	for ($k = 0; ($array[$k] = mysql_fetch_array($result, MYSQL_ASSOC)); $k++)
		;
	unset($array[$k]);
	@mysql_free_result($result);
	return $array;
} /* }}} */

function ezin_get_section_list_counts($lang = '', $current_section_id = null) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$current_section_id = intval($current_section_id);
	$query = 'SELECT id, type, accesses_count, sessions_count, raw_accesses_count, raw_sessions_count'
		.' FROM '.$ezin_sys['db']['tables']['sections']
		.' WHERE type IN ("normal", "global")';
	if ($lang != '') {
		$query .= ' AND (lang IS NULL OR lang = ""'
				.' OR lang LIKE "'.ezin_db_escape($lang).'")';
	}
	$query .= ' ORDER BY ord ASC, type ASC, id ASC';
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_section_list()');
	$array = array();
	for ($k = 0; ($row = mysql_fetch_array($result, MYSQL_ASSOC)); $k++) {
		$array['section_accesses_count_'.$row['id']]     = $row['accesses_count'];
		$array['section_sessions_count_'.$row['id']]     = $row['sessions_count'];
		$array['section_raw_accesses_count_'.$row['id']] = $row['raw_accesses_count'];
		$array['section_raw_sessions_count_'.$row['id']] = $row['raw_sessions_count'];
		if ($row['id'] == $current_section_id) {
			$array['section_selected_'.$row['id']] = 1;
		} else if (!$current_section_id && $row['type'] == 'global') {
			$array['section_selected_'.$row['id']] = 1;
		} else {
			$array['section_selected_'.$row['id']] = 0;
		}
	}
	@mysql_free_result($result);
	return $array;
} /* }}} */

/*
 * ezin_get_section()
 */
function ezin_get_section($section_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$section_id = intval($section_id);

	$query = 'SELECT'
		.' sections.id section_id,'
		.' sections.type section_type,'
		.' sections.ord section_ord,'
		.' sections.a_limit section_limit,'
		.' sections.accesses_count as section_accesses_count,'
		.' sections.sessions_count as section_sessions_count,'
		.' sections.raw_accesses_count as section_raw_accesses_count,'
		.' sections.raw_sessions_count as section_raw_sessions_count,'
		.' sections.name section_name,'
		.' sections.access_key   section_access_key,'
		.' sections.template_key section_template_key,'
		.' sections.link section_link,'
		.' UNIX_TIMESTAMP(editorials.datetime) section_editorial_timestamp,'
		.' editorials.text_wordcount section_editorial_text_wordcount,'
		.' editorials.text_flags     section_editorial_text_flags,'
		.' editorials.text           section_editorial_text,'
		.' editorials.title          section_editorial_title'
		.' FROM '.$ezin_sys['db']['tables']['sections'].' AS sections'
		.' LEFT JOIN '.$ezin_sys['db']['tables']['editorials'].' AS editorials'
		.' ON sections.id = editorials.section_id'
		.' AND (editorials.datetime IS NULL OR editorials.datetime <= SYSDATE())'
		/* XXX let this here, although global sections now use direct ID retrieving */
		.' WHERE '.($section_id > 0 
				? 'sections.id = '.$section_id
				: 'sections.type = "global"')
		.' ORDER BY editorials.datetime DESC LIMIT 1';

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_section() [1]');
	if (($row = mysql_fetch_array($result, MYSQL_ASSOC)) == false)
		$row = array();
	@mysql_free_result($result);

	if (is_null($row['section_editorial_timestamp'])
			&& is_null($row['section_editorial_title'])
			&& is_null($row['section_editorial_text']))
	{
		$query = 'SELECT'
		.' UNIX_TIMESTAMP(editorials.datetime) section_editorial_timestamp,'
		.' editorials.text_wordcount section_editorial_text_wordcount,'
		.' editorials.text_flags     section_editorial_text_flags,'
		.' editorials.text           section_editorial_text,'
		.' editorials.title          section_editorial_title'
		.' FROM '.$ezin_sys['db']['tables']['editorials'].' AS editorials,'
		.' '.$ezin_sys['db']['tables']['sections'].' AS sections'
		.' WHERE sections.id = editorials.section_id AND sections.type = "global"'
		.' AND (editorials.datetime IS NULL OR editorials.datetime <= SYSDATE())'
		.' ORDER BY editorials.datetime DESC LIMIT 1';

		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_get_section() [2]');
		if (($subrow = mysql_fetch_array($result, MYSQL_ASSOC)) != false)
			$row = array_merge($row, $subrow);
		@mysql_free_result($result);
	}

	return $row;
} /* }}} */

/*
 * ezin_get_articles_todo()
 */
function ezin_get_articles_todo() /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$query = sprintf('SELECT COUNT(*) FROM %s WHERE datetime > SYSDATE()',
			$ezin_sys['db']['tables']['articles']);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_articles_todo()');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row = array();
	@mysql_free_result($result);
	return intval($row[0]);
} /* }}} */

/*
 * ezin_get_article_list()
 * TODO: this should probably re-think or re-implement (with database enums too)
 */
function ezin_get_article_list($section_id = 0, $author_id = 0, $limit = 10, $lang = '') /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;
	$section_id = intval($section_id);
	//$author_id  = intval($author_id);
	$limit      = intval($limit);
	$query = 'SELECT'
		.' articles.id article_id,'
		.' articles.atitle article_atitle,'
		.' articles.title article_title,'
		.' articles.status article_status,'
		.' articles.access_key   article_access_key,'
		.' articles.template_key article_template_key,'
		.' UNIX_TIMESTAMP(articles.datetime) article_timestamp,'
		.' articles.bodytext_wordcount article_bodytext_wordcount,'
		.' articles.bodytext_flags     article_bodytext_flags,'
		.' articles.hometext_wordcount article_hometext_wordcount,'
		.' articles.hometext_flags     article_hometext_flags,'
		.' articles.hometext article_hometext,'
		.' authors.id article_author_id,'
		.' authors.name article_author_name,'
		.' authors.email article_author_email,'
		.' authors.homepage article_author_homepage,'
		.' authors.access_key   article_author_access_key,'
		.' authors.template_key article_author_template_key,'
		//.' authors.description article_author_description,'
		.' sections.id article_section_id,'
		.' sections.name article_section_name,'
		.' sections.type article_section_type,'
		.' sections.access_key   article_section_access_key,'
		.' sections.template_key article_section_template_key,'
		.' sections.link article_section_link'
		//.' sections.ord article_section_ord,'
		//.' sections.a_limit article_section_limit'
		.' FROM '.$ezin_sys['db']['tables']['articles'].' AS articles'
		.' LEFT JOIN '.$ezin_sys['db']['tables']['authors'].' AS authors'
		.' ON articles.author_id = authors.id,'
		.' '.$ezin_sys['db']['tables']['sections'].' AS sections';
	$query .=
		' WHERE articles.section_id = sections.id'
		.' AND articles.datetime <= SYSDATE()'
		.' AND UNIX_TIMESTAMP(articles.datetime) > 0'
		.' AND articles.status NOT IN ("", "disabled")'
		.' AND sections.type NOT IN ("", "full_hidden", "disabled")';
	if ($lang != '') {
		$query .= ' AND (lang IS NULL OR lang = ""'
				.' OR lang LIKE "'.ezin_db_escape($lang).'")';
	}
	if (strlen($author_id) > 0) {
		if (is_numeric($author_id)) {
			if (intval($author_id) > 0) {
				$query .= ' AND authors.id = '.intval($author_id);
			}
		} else {
			$query .= ' AND authors.name LIKE "%'.ezin_db_escape($author_id).'%"';
		}
	} else {
		$query .= ' AND articles.status NOT IN ("hidden")';
		if ($section_id > 0) {
			$query .= ' AND sections.id = '.$section_id;
			$query .= ' AND (sections.type != "global" OR articles.status != "partly_showed")';
		} else {
			$query .= ' AND sections.type != "hidden2" AND articles.status != "partly_showed"';
		}
	}
	$query .= ' ORDER BY articles.datetime '
		.($ezin_cfg['order_articles_desc'] ? 'DESC' : 'ASC')
		.', articles.id DESC';
	if ($limit > 0) {
		$query .= ' LIMIT '.$limit;
	}
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_article_list()');
	$array = array();
	for ($k = 0; ($array[$k] = @mysql_fetch_array($result, MYSQL_ASSOC)); $k++)
		;
	unset($array[$k]);
	@mysql_free_result($result);
	return $array;
} /* }}} */

/*
 * ezin_get_article_list_counts()
 */
function ezin_get_article_list_counts($section_id = 0, $author_id = 0, $limit = 10, $lang = '') /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;
	$section_id = intval($section_id);
	$limit      = intval($limit);
	$query = 'SELECT articles.id,'
		.' articles.accesses_count, articles.sessions_count, articles.message_count'
		.' FROM '.$ezin_sys['db']['tables']['articles'].' AS articles'
		.' LEFT JOIN '.$ezin_sys['db']['tables']['authors'].' AS authors'
		.' ON articles.author_id = authors.id,'
		.' '.$ezin_sys['db']['tables']['sections'].' AS sections';
	$query .=
		' WHERE articles.section_id = sections.id'
		.' AND articles.datetime <= SYSDATE()'
		.' AND UNIX_TIMESTAMP(articles.datetime) > 0'
		.' AND articles.status NOT IN ("", "disabled")'
		.' AND sections.type NOT IN ("", "full_hidden", "disabled")';
	if ($lang != '') {
		$query .= ' AND (lang IS NULL OR lang = ""'
				.' OR lang LIKE "'.ezin_db_escape($lang).'")';
	}
	if (strlen($author_id) > 0) {
		if (is_numeric($author_id)) {
			$query .= ' AND authors.id = '.intval($author_id);
		} else {
			$query .= ' AND authors.name LIKE "%'.ezin_db_escape($author_id).'%"';
		}
	} else {
		$query .= ' AND articles.status NOT IN ("hidden")';
		if ($section_id > 0) {
			$query .= ' AND sections.id = '.$section_id;
			$query .= ' AND (sections.type != "global" OR articles.status != "partly_showed")';
		} else {
			$query .= ' AND sections.type != "hidden2" AND articles.status != "partly_showed"';
		}
	}
	$query .= ' ORDER BY articles.datetime '
		.($ezin_cfg['order_articles_desc'] ? 'DESC' : 'ASC')
		.', articles.id DESC';
	if ($limit > 0) {
		$query .= ' LIMIT '.$limit;
	}
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_article_list_counts()');
	$array = array();
	for ($k = 0; ($row = mysql_fetch_array($result, MYSQL_ASSOC)); $k++) {
		$array['article_accesses_count_'.$row['id']] = $row['accesses_count'];
		$array['article_sessions_count_'.$row['id']] = $row['sessions_count'];
		$array['article_message_count_'.$row['id']]  = $row['message_count'];
	}
	@mysql_free_result($result);
	return $array;
} /* }}} */

/*
 * ezin_get_article()
 */
function ezin_get_article($article_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;

	$article_id = intval($article_id);

	$query = 'SELECT'
		.' articles.id article_id,'
		.' articles.author_id article_author_id,'
		.' articles.section_id article_section_id,'
		.' articles.atitle article_atitle,'
		.' articles.title article_title,'
		.' articles.status article_status,'
		.' articles.accesses_count article_accesses_count,'
		.' articles.sessions_count article_sessions_count,'
		.' articles.message_count  article_message_count,'
		.' articles.access_key     article_access_key,'
		.' articles.template_key   article_template_key,'
		.' UNIX_TIMESTAMP(articles.datetime) article_timestamp,'
		.' articles.hometext_wordcount article_hometext_wordcount,'
		.' articles.bodytext_wordcount article_bodytext_wordcount,'
		.' articles.hometext_flags article_hometext_flags,'
		.' articles.bodytext_flags article_bodytext_flags,'
		.' articles.hometext article_hometext,'
		.' articles.bodytext article_bodytext,'
		.' authors.id article_author_id,'
		.' authors.access_key article_author_access_key,'
		.' authors.name article_author_name,'
		.' authors.email article_author_email,'
		.' authors.homepage article_author_homepage,'
		//.' authors.description article_author_description,'
		.' sections.id article_section_id,'
		.' sections.name article_section_name,'
		.' sections.type article_section_type,'
		.' sections.access_key   article_section_access_key,'
		.' sections.template_key article_section_template_key,'
		.' sections.link article_section_link'
		//.' sections.ord article_section_ord,'
		//.' sections.a_limit article_section_limit'
		.' FROM '.$ezin_sys['db']['tables']['articles'].' AS articles'
		.' LEFT JOIN '.$ezin_sys['db']['tables']['authors'].' AS authors'
		.' ON articles.author_id = authors.id,'
		.' '.$ezin_sys['db']['tables']['sections'].' AS sections' 
		.' WHERE articles.section_id = sections.id '
		.' AND articles.id = '.$article_id
		.' ORDER BY articles.datetime '
		.($ezin_cfg['order_articles_desc'] ? 'DESC' : 'ASC')
		.', articles.id ASC';

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_article()');

	if (($row = @mysql_fetch_array($result, MYSQL_ASSOC)) == false)
		$row = array();
	@mysql_free_result($result);

	return $row;
} /* }}} */

/*
 * ezin_get_author()
 */
function ezin_get_author($author_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;

	$author_id = intval($author_id);
	$ret_row   = array();

	$query = sprintf('SELECT * FROM %s WHERE id = %d',
			$ezin_sys['db']['tables']['authors'], $author_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_author() [1]');
	if (($row = @mysql_fetch_array($result, MYSQL_ASSOC)) == false)
		$row = array();
	@mysql_free_result($result);
	foreach (array_keys($row) as $key) {
		$ret_row['author_'.$key] = $row[$key];
	}

	$query = sprintf('SELECT * FROM %s WHERE author_id = %d LIMIT 1',
			$ezin_sys['db']['tables']['users'], $author_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_author() [2]');
	if (($row = @mysql_fetch_array($result, MYSQL_ASSOC)) == false)
		$row = array();
	@mysql_free_result($result);
	foreach (array_keys($row) as $key) {
		if ($key == 'id') {
			$ret_row['author_user_id'] = $row[$key];
		} else {
			$ret_row['author_'.$key] = $row[$key];
		}
	}

	$query = sprintf('SELECT COUNT(*) FROM %s WHERE author_id = %d AND status != "disabled"',
			$ezin_sys['db']['tables']['articles'], $author_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_author() [3]');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row = array();
	@mysql_free_result($result);
	$ret_row['author_article_count'] = intval($row[0]);

	return $ret_row;
} /* }}} */

function ezin_save_author($data) /* {{{ */
{
	global $ezin_sys, $ezin_cfg;
	global $ezin_db_handle;

	$query = sprintf('SELECT COUNT(*) FROM %s WHERE username = "%s"',
			$ezin_sys['db']['tables']['users'], ezin_db_escape($data['username']));
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_save_author() [1]');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row = array();
	@mysql_free_result($result);
	$exists = intval($row[0]);

	if ($exists) {
		return ezin_update_author($data);
	} else if ($ezin_cfg['user_create_role_id'] > 0) {
		return ezin_insert_author($data);
	}

	return false;
} /* }}} */

function ezin_update_author($data) /* {{{ */
{
	global $ezin_sys, $ezin_cfg;
	global $ezin_db_handle;

	$password = $data['password'];
	$data = ezin_db_escape($data);

	// users update (password only)
	if (strlen($password) > 0) {
		$query = sprintf('UPDATE %s SET password="%s" WHERE username="%s"',
				$ezin_sys['db']['tables']['users'], md5($data['password']), $data['username']);
		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_author() [1]');
	}
	// author_id
	$query = sprintf('SELECT author_id FROM %s WHERE username = "%s"',
			$ezin_sys['db']['tables']['users'], $data['username']);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_update_author() [2]');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row = array();
	@mysql_free_result($result);
	$author_id = intval($row[0]);
	// users update
	$query = sprintf('UPDATE %s SET name="%s",email="%s",homepage="%s",description="%s" WHERE id=%d',
			$ezin_sys['db']['tables']['authors'], $data['name'], $data['email'], $data['homepage'],
			$data['description'], $author_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_update_author() [3]');
	return true;
} /* }}} */

function ezin_insert_author($data) /* {{{ */
{
	global $ezin_sys, $ezin_cfg;
	global $ezin_db_handle;

	$data = ezin_db_escape($data);

	// authors INSERT
	$query = sprintf('INSERT INTO %s'
			.' (type,name,email,homepage,description)'
			.' VALUES ("external","%s","%s","%s","%s")',
			$ezin_sys['db']['tables']['authors'],
			$data['name'], $data['email'], $data['homepage'], $data['description']);
	$result = @mysql_query($query, $ezin_db_handle)
		or ezin_db_error('ezin_insert_author() [1]');
	// author_id
	$query = 'SELECT LAST_INSERT_ID()';
	$result = @mysql_query($query, $ezin_db_handle)
		or ezin_db_error('ezin_insert_author() [2]');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row = array();
	@mysql_free_result($result);
	$author_id = intval($row[0]);
	// users INSERT
	$query = sprintf('INSERT INTO %s'
			.' (username,password,author_id,role_id)'
			.' VALUES ("%s","%s",%d,%d)',
			$ezin_sys['db']['tables']['users'],
			$data['username'], md5($data['password']),
			$author_id, $ezin_cfg['user_create_role_id']);
	$result = @mysql_query($query, $ezin_db_handle)
		or ezin_db_error('ezin_insert_author() [3]');
	return true;
} /* }}} */

/*
 * COUNTS MANIPULATION FUNCTIONS
 */

/*
 * ezin_set_author_counts()
 */ 
function ezin_set_author_counts($author_id, $count_ar) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$author_id = intval($author_id);
	$key_array  = array('accesses_count', 'sessions_count');
	$set_str    = '';
	$delim      = '';

	foreach ($key_array as $key) {
		if (isset($count_ar[$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar[$key]);
			if ($delim == '')
				$delim = ', ';
		} elseif (isset($count_ar['author_'.$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar['author_'.$key]);
			if ($delim == '')
				$delim = ', ';
		}
	}

	if ($set_str == '')
		return true;

	$query = 'UPDATE '.$ezin_sys['db']['tables']['authors']
		.' SET '.$set_str
		.' WHERE id = '.$author_id;

	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_set_author_counts()');

	return true;
} /* }}} */

/*
 * ezin_set_section_counts()
 */ 
function ezin_set_section_counts($section_id, $count_ar) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$section_id = intval($section_id);
	$key_array  = array('accesses_count', 'sessions_count',
			'raw_accesses_count', 'raw_sessions_count');
	$set_str    = '';
	$delim      = '';

	foreach ($key_array as $key) {
		if (isset($count_ar[$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar[$key]);
			if ($delim == '')
				$delim = ', ';
		} elseif (isset($count_ar['section_'.$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar['section_'.$key]);
			if ($delim == '')
				$delim = ', ';
		}
	}

	if ($set_str == '')
		return true;

	$query = 'UPDATE '.$ezin_sys['db']['tables']['sections']
		.' SET '.$set_str
		.' WHERE id = '.$section_id;

	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_set_section_counts()');

	return true;
} /* }}} */

/*
 * ezin_set_article_counts()
 */ 
function ezin_set_article_counts($article_id, $count_ar) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$article_id = intval($article_id);
	$key_array  = array('accesses_count', 'sessions_count');
	$set_str    = '';
	$delim      = '';

	foreach ($key_array as $key) {
		if (isset($count_ar[$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar[$key]);
			if ($delim == '')
				$delim = ', ';
		} elseif (isset($count_ar['article_'.$key])) {
			$set_str .= $delim.$key.' = '.intval($count_ar['article_'.$key]);
			if ($delim == '')
				$delim = ', ';
		}
	}

	if ($set_str == '')
		return true;

	$query = 'UPDATE '.$ezin_sys['db']['tables']['articles']
		.' SET '.$set_str
		.' WHERE id = '.$article_id;

	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_set_article_counts()');

	return true;
} /* }}} */

/*
 * ezin_update_message_count()
 */ 

function ezin_update_message_count($article_id, $direction = 0, $count = -1) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$article_id = intval($article_id);
	$direction  = intval($direction);
	$count      = intval($count);

	if ($article_id <= 0) {
		return false;
	}

	if ($direction == 0 && $count < 0) {
		$count = ezin_get_message_count($article_id);
	}

	$query = 'UPDATE '.$ezin_sys['db']['tables']['articles']
		.' SET message_count = '
		. ($direction == 0
				? $count
				: 'message_count '.($direction < 0 ? '-' : '+').' '.$count)
		.' WHERE id = '.$article_id;

	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_update_message_count()');

	return true;
} /* }}} */

/*
 * ezin_get_message_count()
 *
 * If article message_count is NULL or negative value, this function is used
 * for setting up this field to proper value.
 */

function ezin_get_message_count($article_id = 0) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$query = 'SELECT count(*)'
		.' FROM '.$ezin_sys['db']['tables']['messages'];

	$article_id = intval($article_id);
	if ($article_id > 0)
		$query .= ' WHERE article_id = '.$article_id;

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_message_count()');

	if (($row = mysql_fetch_array($result, MYSQL_NUM)) === false)
		$row[0] = 0;
	@mysql_free_result($result);

	return $row[0];
} /* }}} */

/*
 * ezin_update_counts_temp()
 */

function ezin_update_counts_temp() /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;
	$query = 'SELECT ip FROM '.$ezin_sys['db']['tables']['counts_temp']
		.' GROUP BY ip'
		.' HAVING UNIX_TIMESTAMP(SYSDATE()) - MAX(timestamp) > '
		.intval($ezin_cfg['session_timeout']);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_update_counts_temp() [1]');
	$delim = '';
	$ips   = '';
	while ($row = mysql_fetch_array($result, MYSQL_NUM)) {
		$ips .= $delim.'"'.ezin_db_escape($row[0]).'"';
		if ($delim == '')
			$delim = ',';
	}
	@mysql_free_result($result);
	if ($ips != '') {
		$query = 'DELETE FROM '.$ezin_sys['db']['tables']['counts_temp']
			.' WHERE ip IN ('.$ips.')'; // )

		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts_temp() [2]');
	}
	return true;
} /* }}} */

/*
 * ezin_update_counts()
 */
function ezin_update_counts($author_id, $section_id, $article_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$author_id    = intval($author_id);
	$section_id   = intval($section_id);
	$article_id   = intval($article_id);
	$remote_addr  = ezin_db_escape($_SERVER['REMOTE_ADDR']);
	$type_ids     = array();
	$all_type_ids = array();
	if ($author_id > 0) {
		$all_type_ids['U'] = '"U'.$author_id.'"';
	} else {
		$all_type_ids['s'] = '"s'.$section_id.'"';
		if ($article_id > 0) {
			$all_type_ids['A'] = '"A'.$article_id.'"';
		} else {
			$all_type_ids['S'] = '"S'.$section_id.'"';
		}
	}
	$query = 'SELECT type_id FROM '.$ezin_sys['db']['tables']['counts_temp']
		.' WHERE ip = "'.$remote_addr.'"'
		.' AND type_id IN ('.join(',',$all_type_ids).')'; // )
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_update_counts() [1]');
	while ($row = @mysql_fetch_row($result)) {
		$type_ids[$row[0][0]] = '"'.$row[0].'"';
	}
	@mysql_free_result($result);
	if ($article_id > 0) {
		$query = 'UPDATE '.$ezin_sys['db']['tables']['articles']
			.' SET accesses_count = accesses_count + 1'
			.(! isset($type_ids['A']) ? ', sessions_count = sessions_count + 1' : '')
			.' WHERE id = '.$article_id;
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts() [2Aa]');
		$query = 'UPDATE '.$ezin_sys['db']['tables']['sections']
			.' SET raw_accesses_count = raw_accesses_count + 1'
			.(! isset($type_ids['s']) ? ', raw_sessions_count = raw_sessions_count + 1' : '')
			.' WHERE id = '.$section_id;
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts() [2Ab]');
	} elseif ($section_id > 0) {
		$query = 'UPDATE '.$ezin_sys['db']['tables']['sections']
			.' SET accesses_count = accesses_count + 1,'
			.' raw_accesses_count = raw_accesses_count + 1'
			.(! isset($type_ids['S']) ? ', sessions_count = sessions_count + 1' : '')
			.(! isset($type_ids['s']) ? ', raw_sessions_count = raw_sessions_count + 1' : '')
			.' WHERE id = '.$section_id;
			//.' WHERE '.($section_id > 0 ? 'id = "'.$section_id.'"' : 'type = "global"');
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts() [2B]');
	} elseif ($author_id > 0) {
		$query = 'UPDATE '.$ezin_sys['db']['tables']['authors']
			.' SET accesses_count = accesses_count + 1'
			.(! isset($type_ids['U']) ? ', sessions_count = sessions_count + 1' : '')
			.' WHERE id = '.$author_id;
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts() [2C]');
	}
	$type_ids = array_diff($all_type_ids, $type_ids);
	if (count($type_ids)) {
		$delim = '';
		$query = 'INSERT INTO '.$ezin_sys['db']['tables']['counts_temp']
			.' (type_id, timestamp, ip) VALUES ';
		foreach ($type_ids as $val) {
			$query .= $delim.'('.$val.',UNIX_TIMESTAMP(SYSDATE()),"'.$remote_addr.'")';
			if ($delim == '')
				$delim = ', ';
		}
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_update_counts() [3A]');
	} else {
		/* If no INSERT action is taken, we will update 's'.
		   Raw session ('s') is already in table and it is present
		   in both cases we must do this job, to avoid unwanted
		   session expiration (imagine situation, when someone
		   will click still on the same article) */
		$type_id = null;
		if (isset($all_type_ids['s'])) {
			$type_id = $all_type_ids['s'];
		} else if (isset($all_type_ids['U'])) {
			$type_id = $all_type_ids['U'];
		}
		if ($type_id !== null) {
			$query = 'UPDATE '.$ezin_sys['db']['tables']['counts_temp']
				.' SET timestamp = UNIX_TIMESTAMP(SYSDATE())'
				.' WHERE ip = "'.$remote_addr.'"'
				.' AND type_id = '.$type_id;
			@mysql_query($query, $ezin_db_handle) 
				or ezin_db_error('ezin_update_counts() [3B]');
		}
	}
} /* }}} */

/*
 * MESSAGES RELATED FUNCTION
 */

/*
 * ezin_get_message_list()
 */

function ezin_get_message_list($article_id, $parent) /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;
	$article_id = intval($article_id);
	$parent     = intval($parent);
	$query = 'SELECT'
		.' messages.id message_id,'
		.' messages.parent message_parent,'
		.' messages.article_id message_article_id,'
		.' messages.author message_author,'
		.' messages.email message_email,'
		.' messages.homepage message_homepage,'
		.' messages.subject message_subject,'
		.' messages.message message_message,'
		.' UNIX_TIMESTAMP(messages.datetime) message_timestamp,'
		.' messages.ip message_ip'
		.' FROM '.$ezin_sys['db']['tables']['messages'].' AS messages '
		.' WHERE messages.parent = '.$parent
		.(isset($article_id) ? ' AND messages.article_id = '.$article_id : '')
		.' ORDER BY messages.datetime '
		. ($ezin_cfg['order_messages_desc'] ? 'DESC' : 'ASC');
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_message_list()');
	$array = array();
	for ($k = 0; ($array[$k] = mysql_fetch_array($result, MYSQL_ASSOC)); $k++)
		;
	unset($array[$k]);
	@mysql_free_result($result);
	return $array;
} /* }}} */

/*
 * ezin_get_message()
 */

function ezin_get_message($message_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$message_id = intval($message_id);

	$query = 'SELECT'
		.' messages.id message_id,'
		.' messages.parent message_parent,'
		.' messages.article_id message_article_id,'
		.' messages.author message_author,'
		.' messages.email message_email,'
		.' messages.homepage message_homepage,'
		.' messages.subject message_subject,'
		.' messages.message message_message,'
		.' UNIX_TIMESTAMP(messages.datetime) message_timestamp,'
		.' messages.ip message_ip,'
		.' articles.id message_article_id,'
		.' articles.author_id message_article_author_id,'
		.' articles.section_id message_article_section_id,'
		.' articles.atitle message_article_atitle,'
		.' articles.title message_article_title,'
		.' articles.status message_article_status,'
		.' UNIX_TIMESTAMP(articles.datetime) message_article_timestamp,'
		.' articles.hometext_wordcount message_article_hometext_wordcount,'
		.' articles.bodytext_wordcount message_article_bodytext_wordcount,'
		.' articles.hometext_flags message_article_hometext_flags,'
		.' articles.bodytext_flags message_article_bodytext_flags,'
		.' articles.hometext message_article_hometext,'
		.' articles.bodytext message_article_bodytext,'
		.' authors.id article_author_id,'
		.' authors.name article_author_name,'
		.' authors.email article_author_email,'
		.' authors.homepage article_author_homepage,'
		//.' authors.description article_author_description,'
		.' sections.id article_section_id,'
		.' sections.name article_section_name,'
		.' sections.type article_section_type'
		//.' sections.ord article_section_ord,'
		//.' sections.a_limit article_section_limit'
		// XXX: article/section access/template keys are missing
		.' FROM '.$ezin_sys['db']['tables']['articles'].' AS articles, '
		.$ezin_sys['db']['tables']['authors'].' AS authors, '
		.$ezin_sys['db']['tables']['sections'].' AS sections, ' 
		.$ezin_sys['db']['tables']['messages'].' AS messages' 
		.' WHERE articles.author_id = authors.id '
		.' AND articles.section_id = sections.id '
		.' AND articles.id = messages.article_id '
		.' AND messages.id = '.$message_id;

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_message()');

	if (($row = @mysql_fetch_array($result, MYSQL_ASSOC)) == false)
		$row = array();
	@mysql_free_result($result);

	return $row;
} /* }}} */

/*
 * ezin_save_message()
 */
function ezin_save_message($data) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$ar = ezin_validate_IDs(array(
				'section_id' => 0,
				'article_id' => 0,
				'message_id' => $data['parent']));
	$article_id = $ar['article_id'];
	if (empty($article_id)) {
		$ar = ezin_validate_IDs(array(
					'section_id' => 0,
					'article_id' => $data['article_id'],
					'message_id' => 0));
		$article_id = $ar['article_id'];
		if (empty($article_id)) {
			return false;
		}
		$data['article_id'] = $article_id;
		$data['parent']     = 0;
	}
	
	$data['ip'] = array();
	foreach (array('REMOTE_HOST', 'REMOTE_ADDR', 'HTTP_X_FORWARDED_FOR') as $key) {
		if (isset($_SERVER[$key]) && strlen($_SERVER[$key]) > 0) {
			$data['ip'][] = $_SERVER[$key];
		}
	}
	$data['ip'] = join(' / ', $data['ip']);
	$data       = ezin_db_escape($data);

	$query = sprintf('INSERT INTO %s'
			.' (id,parent,article_id,author,email,homepage,subject,message,datetime,ip)'
			.' VALUES (0,"%s","%s","%s","%s","%s","%s","%s",SYSDATE(),"%s")',
			$ezin_sys['db']['tables']['messages'],
			$data['parent'], $data['article_id'], $data['author'], $data['email'],
			$data['homepage'], $data['subject'], $data['message'], $data['ip']);

	// $result = must be here, see return below
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_save_message()');

	return $result;
} /* }}} */

/*
 * ezin_check_message_flood()
 */

function ezin_check_message_flood() /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_db_handle;

	$timeout = intval($ezin_cfg['message_flood_timeout']);
	if ($timeout <= 0)
		return 0; // no flood

	$query = 'SELECT count(*) '
		.' FROM ' . $ezin_sys['db']['tables']['messages']
		.' WHERE ip LIKE "'.ezin_db_escape($_SERVER['REMOTE_ADDR']).'%"'
		.' AND UNIX_TIMESTAMP(SYSDATE()) - UNIX_TIMESTAMP(datetime)'
		.' <= '.$timeout;

	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_check_message_flood()');

	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) == false)
		$row[0] = 0;
	@mysql_free_result($result);

	return $row[0]; // 0 - no flooder detected, > 0 - flooders count
} /* }}} */

/*
 * POLL RELATED FUNCTIONS
 */

function ezin_get_poll($section_id, $lang = '') /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$section_id = intval($section_id);

	if ($section_id > 0) {
		$query = 'SELECT'
			.' polls.id poll_id,'
			.' polls.votes poll_votes,'
			.' polls.answers poll_answers,'
			.' polls.question poll_question,'
			.' polls.description poll_description'
				.' FROM '.$ezin_sys['db']['tables']['polls'].' AS polls'
				.' WHERE polls.section_id = '.$section_id
				.' AND polls.dt_valid_from <= SYSDATE()'
				.' AND polls.dt_valid_to   >= SYSDATE()'
				.' ORDER BY polls.dt_valid_from DESC LIMIT 1';
		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_get_poll() [1]');
		$row = mysql_fetch_array($result, MYSQL_ASSOC);
		@mysql_free_result($result);
		if (is_array($row)) {
			return $row;
		}
	}

	$query = sprintf('SELECT id FROM %s WHERE type = "global"', $ezin_sys['db']['tables']['sections']);
	if (strlen($lang) > 0) {
		$query .= sprintf(' AND lang = "%s"', ezin_db_escape($lang));
	}
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_poll() [2]');
	$ids_str = '';
	while ($row = @mysql_fetch_array($result, MYSQL_NUM)) {
		$ids_str != '' && $ids_str .= ',';
		$ids_str .= $row[0];
	}
	@mysql_free_result($result);

	if (strlen($ids_str)) {
		$query = 'SELECT'
			.' polls.id poll_id,'
			.' polls.votes poll_votes,'
			.' polls.answers poll_answers,'
			.' polls.question poll_question,'
			.' polls.description poll_description'
			.' FROM '.$ezin_sys['db']['tables']['polls'].' AS polls'
			.' WHERE polls.section_id IN ('.$ids_str.')'
			.' AND polls.dt_valid_from <= SYSDATE()'
			.' AND polls.dt_valid_to   >= SYSDATE()'
			.' ORDER BY polls.dt_valid_from DESC LIMIT 1';
		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_get_poll() [3]');
		$row = mysql_fetch_array($result, MYSQL_ASSOC);
		@mysql_free_result($result);
		if (is_array($row)) {
			return $row;
		}
	}
	return array(
		'poll_id'       => 0,
		'poll_votes'    => 0,
		'poll_answers'  => 0,
		'poll_question' => ''
		);
} /* }}} */

function ezin_get_poll_answer_list($poll_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	
	$poll_id = intval($poll_id);

	$query = 'SELECT'
		.' id     poll_answer_id,'
		.' answer poll_answer,'
		.' answer poll_answer_answer,'
		.' votes  poll_answer_votes'
		.' FROM '.$ezin_sys['db']['tables']['poll_answers']
		.' WHERE poll_id = '.$poll_id;
	$query .= ' ORDER BY votes DESC, id ASC';
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_poll_answer_list()');
	$array = array();
	$total_votes = 0;
	for ($k = 0; ($array[$k] = mysql_fetch_array($result, MYSQL_ASSOC)); $k++) {
		$total_votes += $array[$k]['poll_answer_votes'];
	}
	unset($array[$k]);
	for ($k = 0; $k < count($array); $k++) {
		$array[$k]['poll_answer_ord'] = $k + 1;
		$array[$k]['poll_answer_pct'] = round(
				($array[$k]['poll_answer_votes'] / max(1, $total_votes)) * 100);
	}
	@mysql_free_result($result);
	return $array;
} /* }}} */

function ezin_get_poll_answer_list_counts($poll_id) /* {{{ */
{
	// TODO implement
	return array();
} /* }}} */

function ezin_is_valid_poll($poll_answer_id) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$poll_answer_id = intval($poll_answer_id);
	$query = 'SELECT answers.poll_id'
		.' FROM '.$ezin_sys['db']['tables']['polls'].' AS polls,'
		.$ezin_sys['db']['tables']['poll_answers'].' AS answers'
		.' WHERE polls.id = answers.poll_id'
		.' AND polls.dt_valid_from <= SYSDATE()'
		.' AND polls.dt_valid_to   >= SYSDATE()'
		.' AND answers.id = '.$poll_answer_id;
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_is_valid_poll()');
	$row = mysql_fetch_array($result, MYSQL_NUM);
	@mysql_free_result($result);
	return is_array($row) ? $row[0] : 0;
} /* }}} */

function ezin_poll_answer_vote($poll_answer_id, $poll_id = 0) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$poll_answer_id = intval($poll_answer_id);
	$poll_id = intval($poll_id);
	$query = sprintf('UPDATE %s SET votes = votes + 1 WHERE id = %d',
			$ezin_sys['db']['tables']['poll_answers'], $poll_answer_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_poll_answer_vote() [1]');
	if ($poll_id > 0) {
		$query = sprintf('UPDATE %s SET votes = votes + 1 WHERE id = %d',
				$ezin_sys['db']['tables']['polls'], $poll_id);
		$result = @mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_poll_answer_vote() [2]');
	}
	return true;
} /* }}} */

/*
 * FEEDBACK/NEWSLETTER RELATED FUNCTIONS
 */

/*
 * ezin_save_feedback()
 */

function ezin_save_feedback($section_id, $data) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$ip = array();
	foreach (array('REMOTE_HOST', 'REMOTE_ADDR', 'HTTP_X_FORWARDED_FOR') as $key) {
		if (isset($_SERVER[$key]) && strlen($_SERVER[$key]) > 0) {
			$ip[] = $_SERVER[$key];
		}
	}
	$ip = stripslashes(join(' / ', $ip));
	$ua = stripslashes(@$_SERVER['HTTP_USER_AGENT']);

	$query = sprintf('INSERT INTO %s (id,datetime,section_id,ip,user_agent)'
			.' VALUES (0,SYSDATE(),%d,"%s","%s")',
			$ezin_sys['db']['tables']['feedbacks'], intval($section_id),
			ezin_db_escape($ip), ezin_db_escape($ua));
	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_save_feedback() [1]');

	$query   = 'SELECT LAST_INSERT_ID()';
	$result  = @mysql_query($query, $ezin_db_handle);
	$feed_id = 0;
	if ($result != false) {
		$feed_id = @mysql_fetch_array($result, MYSQL_NUM);
		$feed_id = intval(@$feed_id[0]);
		@mysql_free_result();
	}
	if ($feed_id <= 0) {
		ezin_db_error('ezin_save_feedback() [2]');
	}

	foreach ($data as $key => $val) {
		$query = sprintf('INSERT INTO %s (feedback_id, field, data)'
				.' VALUES (%d, "%s", "%s")',
				$ezin_sys['db']['tables']['feedback_data'], $feed_id,
				ezin_db_escape($key), ezin_db_escape($val));
		@mysql_query($query, $ezin_db_handle) 
			or ezin_db_error('ezin_save_feedback() [3]'."[$key]");
	}

	return true;
} /* }}} */

function ezin_save_email($section_id, $email) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	$ip = array();
	foreach (array('REMOTE_ADDR', 'HTTP_X_FORWARDED_FOR') as $key) {
		isset($_SERVER[$key]) && $ip[] = $_SERVER[$key];
	}
	$ip = stripslashes(join(' / ', $ip));
	$ua = stripslashes(@$_SERVER['HTTP_USER_AGENT']);

	$query = sprintf('INSERT INTO %s (email,id,dt_added,section_id,ip,user_agent)'
			.' VALUES ("%s",0,SYSDATE(),%d,"%s","%s")',
			$ezin_sys['db']['tables']['nl_emails'], ezin_db_escape($email),
			intval($section_id), ezin_db_escape($ip), ezin_db_escape($ua));
	if (@mysql_query($query, $ezin_db_handle)) {
		return true;
	}
	return false;
} /* }}} */

/*
 * CACHE RELATED FUNCTIONS
 */

function ezin_get_config() /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	// Config retrieval from cache
	$content = ezin_cache_load_content('config');
	if (isset($content)) {
		$array = @unserialize($content);
		if (is_array($array) && count($array) > 0) {
			return $array;
		} else {
			// for case that stored content was corrupted
			ezin_cache_clear('config');
		}
	}

	// Config retrieval and parsing
	$query = sprintf('SELECT a_key, type, value FROM %s',
			$ezin_sys['db']['tables']['config_data']);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_config() [v2:1]');
	$array = array();
	while ($row = @mysql_fetch_array($result, MYSQL_ASSOC)) {
		if (isset($row['type']) && $row['type'] == 1) {
			$array[$row['a_key']] = ezin_eval($row['value']);
		} else {
			$array[$row['a_key']] = $row['value'];
		}
	}
	@mysql_free_result($result);

	if (! is_array($array) || count($array) <= 0) {
		ezin_db_error('configuration not initialized');
	}

	/* Move this to the more proper (better) place and remove it from "config".
	   -- Nepto [2006-07-09] */
	$query = sprintf('SELECT COUNT(*), SUM(id) FROM %s',
			$ezin_sys['db']['tables']['authors']);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_get_config() [v2:2]');
	if (($row = @mysql_fetch_array($result, MYSQL_NUM)) && $row[0] <= 1) {
		$array['single_author_id'] = intval($row[1]); // single author ID
	} else {
		$array['single_author_id'] = -1;
	}

	// Config storage in cache
	ezin_cache_save('config', serialize($array));

	return $array;
} /* }}} */

function ezin_cache_save($cache_id = 0, $content = '', $var_var = 0) /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$cache_id = ezin_db_escape(trim($cache_id));
	$content  = ezin_db_escape($content);
	$var_var  = intval($var_var);
	$query    = sprintf('INSERT INTO %s (id, var_var, timestamp, content)'
			.' VALUES ("%s", %d, UNIX_TIMESTAMP(SYSDATE()), \'%s\')',
			$ezin_sys['db']['tables']['cache'], $cache_id, $var_var, $content);
	// possible error is ignored here
	@mysql_query($query, $ezin_db_handle);
	return true;
} /* }}} */

function ezin_cache_load($cache_id = 'X') /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$cache_id = ezin_db_escape(trim($cache_id));
	$var_var  = 0;
	$query    = sprintf('SELECT * FROM %s WHERE id = "%s"',
			$ezin_sys['db']['tables']['cache'], $cache_id);
	$result = @mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_cache_load()');
	$row = @mysql_fetch_array($result, MYSQL_ASSOC);
	@mysql_free_result($result);
	if ($row === false) {
		return false;
	}
	return $row;
} /* }}} */

function ezin_cache_load_content($cache_id = 'X') /* {{{ */
{
	$ar = ezin_cache_load($cache_id);
	if (isset($ar['content'])) {
		return $ar['content'];
	}
	return false;
} /* }}} */

function ezin_cache_clear($cache_id = 'X') /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;
	$delete_all = false;
	$delete_gs  = false;
	if (is_array($cache_id)) {
		if (count($cache_id) <= 0) {
			$delete_all = true;
		} else {
			$cache_id_list = array();
			foreach ($cache_id as $id) {
				if ($id == 'Sglobal') {
					$delete_gs = true;
					continue;
				}
				$cache_id_list[] = "id LIKE '".ezin_db_escape(trim($id))."%'";
			}
			$query = sprintf('DELETE FROM %s WHERE %s',
					$ezin_sys['db']['tables']['cache'],
					join(' OR ', $cache_id_list));
		}
	} else {
		if (strlen($cache_id) <= 0) {
			$delete_all = true;
		} else if ($cache_id == 'Sglobal') {
			$delete_gs  = true;
		} else {
			$cache_id = ezin_db_escape(trim($cache_id));
			$query = sprintf('DELETE FROM %s WHERE id LIKE "%s%%"',
					$ezin_sys['db']['tables']['cache'], $cache_id);
		}
	}
	if ($delete_all) {
		$query = 'DELETE FROM '.$ezin_sys['db']['tables']['cache'];
	} else if ($delete_gs) {
		$query2 = sprintf('SELECT id FROM %s WHERE type = "global"', $ezin_sys['db']['tables']['sections']);
		$result = @mysql_query($query2, $ezin_db_handle) 
			or ezin_db_error('ezin_cache_clear() [1]');
		$global_sections = array();
		while ($row = @mysql_fetch_array($result, MYSQL_NUM)) {
			$global_sections[] = 'S'.$row[0];
		}
		@mysql_free_result($result);
		$ret = count($global_sections) > 0 ? ezin_cache_clear($global_sections) : true;
		if ($ret === true && strlen($query) > 0) {
		} else {
			return $ret;
		}
	}
	@mysql_query($query, $ezin_db_handle) 
		or ezin_db_error('ezin_cache_clear() [2]');
	return true;
} /* }}} */

/* Modeline for ViM {{{
 * vim: set ts=4:
 * vim600: fdm=marker fdl=0 fdc=0:
 * }}} */

?>
