<?php

/*
 * Metafox - enterprise content management system
 *
 * db-core.inc.php - DB handle and DB error manipulation interface
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@platon.sk>
 * Copyright (c) 2001-2006 Platon Group, http://platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://platon.sk/projects/Metafox/
 */

/* $Platon: Metafox/lib/db-core/db-core.inc.php,v 1.10 2007-02-24 23:43:50 nepto Exp $ */

/* MAINTAINANCE: This file is maintained in Metafox project and distributed
   across the particular projects accordingly. */

/*
 * Metafox global variabiles
 */

global $ezin_db_handle;
$ezin_db_handle = null;

/*
 * ezin_db_error()
 *
 * Database error handling function
 */

function ezin_db_error($fnc_name = '', $message = '') /* {{{ */
{
	if (strlen($message) <= 0) {
		$message = 'error during work with database server';
	}

	if (strlen($fnc_name) > 0) {
		$message = "$fnc_name: $message";
	}

	if (0) {
		die ('<b>Metafox fatal failure</b>: ' . $message 
				. '<br>Please contact webmaster or system administrator'
				. ' of this page with this problem.<br>');
	} else {
		global $ezin_db_handle;
		if (is_resource($ezin_db_handle) && mysql_error($ezin_db_handle)) {
			$message .= "\n";
			$message .= 'ERROR #'.mysql_errno($ezin_db_handle).': '.mysql_error($ezin_db_handle);
		}
		trigger_error($message, E_USER_ERROR);
	}

	return false;
} /* }}} */

/*
 * ezin_open_db_connection()
 *
 * Opens database connection.
 */

function ezin_open_db_connection() /* {{{ */
{
	global $ezin_sys;
	global $ezin_db_handle;

	if (is_resource($ezin_db_handle)) {
		return $ezin_db_handle;
	}

	$ezin_db_handle = @mysql_pconnect(
			$ezin_sys['db']['host'].':'.$ezin_sys['db']['port'], 
			$ezin_sys['db']['user'], $ezin_sys['db']['pass']);

	if ($ezin_db_handle == false)
		return false;

	if (@mysql_select_db($ezin_sys['db']['name'], $ezin_db_handle) == false)
		return false;

	return $ezin_db_handle;
} /* }}} */


/*
 * ezin_close_db_connection()
 *
 * Closes database connection.
 */

function ezin_close_db_connection() /* {{{ */
{
	global $ezin_db_handle;
	$ret = @mysql_close($ezin_db_handle);
	$ezin_db_handle = null;
	return $ret; // true if OK, false if failed
} /* }}} */


/*
 * ezin_check_db_handle()
 *
 * Check if databse connection is still alive.
 */

function ezin_check_db_handle() /* {{{ */
{
	global $ezin_db_handle;
	if (is_null($ezin_db_handle)) {
		ezin_open_db_connection() 
			or ezin_db_error('ezin_check_db_handle()', 'database server connection error');
	}
} /* }}} */

/*
 * ezin_set_db_encoding()
 *
 * Set-up data retrieval encoding
 */

function ezin_set_db_encoding($encoding = '') /* {{{ */
{
	global $ezin_db_handle;
	ezin_check_db_handle();
	$client_encoding = mysql_client_encoding($ezin_db_handle);
	if (! strcasecmp($client_encoding, 'latin1')) {
		return; // do rather nothing when LATIN1 (old MySQL 3.23 tables)
	}
	static $enc_map = array(
			'UTF8'   => array('utf', 'utf8'),
			'CP1250' => array('windows', 'windows1250', 'cp1250'),
			'LATIN2' => array('latin', 'latin2', 'iso88592'),
			''       => array()
			);
	if (strlen($encoding) <= 0) {
		global $ezin_cfg;
		$encoding = @$ezin_cfg['encoding'];
	}
	$encoding = trim($encoding);
	$encoding = strtolower($encoding);
	$encoding = preg_replace('|[_-]|', '', $encoding);
	foreach ($enc_map as $mysql_encoding => $ar) {
		if (in_array($encoding, $ar)) {
			break;
		}
	}
	if (strlen($mysql_encoding) > 0) {
		mysql_query('SET NAMES '.$mysql_encoding, $ezin_db_handle);
		mysql_query('SET COLLATION_CONNECTION='.$mysql_encoding.'_GENERAL_CI', $ezin_db_handle);
	}
	return;
} /* }}} */

/*
 * ezin_db_data_escape()
 *
 * Escape database data
 */

function ezin_db_escape($data) /* {{{ */
{
	static $escape_function = null;
	if (is_null($escape_function)) {
		foreach (array('mysql_escape_string', 'addslashes') as $fnc_name) {
			if (function_exists($fnc_name)) {
				$escape_function = $fnc_name;
				break;
			}
		}
		if (is_null($escape_function)) {
			die('ezin_db_escape(): unable to set DB escape function');
		}
	}
	if (is_array($data)) {
		foreach (array_keys($data) as $key) {
			$data[$key] = $escape_function($data[$key]);
		}
	} else {
		$data = $escape_function($data);
	}
	return $data;
} /* }}} */


/* Modeline for ViM {{{
 * vim: set ts=4:
 * vim600: fdm=marker fdl=0 fdc=0:
 * }}} */

?>
